//
//  GLTrackSourceList.h
//  GLGPSKit
//
//  Created by Jeffrey Early on 2/3/10.
//  Copyright 2010 Early Innovations, LLC. All rights reserved.
//

#import <Cocoa/Cocoa.h>

// This object represents the model of the entire source list tree, but also includes tracks and waypoints
// associated with the selected source list (which *could* be separated to a different object).
// On initialization it creates the file and date trees automatically, but doesn't add any smart files.

@class  GLDateTree, GLFileTree, GLSmartFileTree, GLSmartFile;
@interface GLTrackSourceList : NSTreeController

/************************************************/
/*		Source Tree								*/
/************************************************/

#pragma mark -
#pragma mark Source Tree
#pragma mark

// The root of the tree...
@property(nonatomic, strong) NSMutableArray *sources;

// ...which contains these three children nodes
@property(nonatomic, strong) GLDateTree *dateTree;
@property(nonatomic, strong) GLFileTree *fileTree;
@property(nonatomic, strong) GLSmartFileTree *smartTree;

// The date and file trees are built automatically, but we get to add as many smart files as we want.
- (void) addSmartFile: (GLSmartFile *) smartFile;
- (void) removeSmartFile: (GLSmartFile *) smartFile;

// This will delete the tracks and waypoints listed in source lists from the date and smart trees.
// For the file tree, this will also delete the import source object.
- (void) deleteObjectsFromSourceList: (id) sourceList;

/************************************************/
/*		Source Tree Selection					*/
/************************************************/

#pragma mark -
#pragma mark Source Tree Selection
#pragma mark

// At this point, we only allow one selection.
@property(nonatomic, strong) NSArray *sourceListSelectionIndexPaths;
@property(nonatomic, strong) id selectedSourceList;

// Returns the source list at the given index path if you hand it the root array
- (id) sourceListInChildrenArray: (NSArray *) array atIndexPath: (NSIndexPath *) path;

// Returns a path with either a) the indexes in the reverse order or b) nil, of the source list path.
- (NSIndexPath *) indexPathOfSourceList: (id) sourceList inChildrenArray: (NSArray *) array;

/************************************************/
/*		Tracks & Waypoints						*/
/************************************************/

#pragma mark -
#pragma mark Tracks & Waypoints
#pragma mark

// These are the tracks and waypoints associated with the current source tree selection.

@property(nonatomic, strong) NSMutableArray *tracksAndWaypoints;
@property(nonatomic, strong) NSMutableArray *tracksAndWaypointsSortDescriptors;
@property(nonatomic, strong) NSIndexSet *tracksAndWaypointsSelectionIndexes;
@property(readonly) NSArray *selectedTracksAndWaypoints;
- (BOOL) removeSelectedTracksAndWaypoints: (NSArray *) objects;
- (BOOL) addSelectedTracksAndWaypoints: (NSArray *) objects;

@property NSUInteger countOfTracks;
@property NSUInteger countOfTrackPoints;
@property NSUInteger countOfWaypoints;

// Deletes the track or waypoint
- (void) deleteTrackOrWaypoint: (id) object;

// Refreshes the tracksAndWaypoints array (for private use, obviously). This can happen for a couple of reasons,
// 1. The selected source list changes
// 2. The selected source list issues a @"shouldRefreshTracksAndWaypoints" because something else (like importing) modified its values.
- (void) refreshTracksAndWaypoints;

@end
